
# q-expansion of newform 117.4.q.c, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 117
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[92, [1, 0]], [28, [0, 1]]]
aps_data = [[2, 2], [0, 0], [8, -16], [26, -13], [13, 13], [-39, 52], [0, 27], [-102, 51], [57, -57], [-69, 69], [42, -84], [-23, -23], [227, 227], [0, 85], [-198, 396], [-426, 0], [22, -11], [0, 17], [95, 95], [-674, 337], [-580, 1160], [-1244, 0], [-246, 492], [-177, -177], [1426, -713], [1959, -1959], [1856, 0], [-255, 255], [352, -704], [0, 411], [2243, -2243], [372, 0], [1374, -687], [0, 2545], [-1506, 753], [50, -100], [-1534, 0], [-1886, 943], [-939, -939], [0, -873], [-1287, 1287], [2, 0], [0, -2841], [-2451, -2451], [-1589, -1589], [0, 1685], [-1681, 1681], [-2365, -2365], [-506, 253], [-104, 208], [5778, 0], [-1074, 2148], [2378, -1189], [0, 4491], [5451, -5451], [-783, 783], [0, -5085], [-765, -765], [0, 3421], [468, -936], [-7177, 7177], [-10754, 5377], [2758, -5516], [-6192, 0], [-770, 0], [4652, -9304], [-6094, 3047], [8278, 0], [0, 6867], [7017, 7017], [-3353, -3353], [774, -1548], [-3665, 3665], [0, -5371], [-6645, -6645], [-2794, 1397], [9858, 0], [-10070, 5035], [4379, 4379], [4970, -2485], [5397, -5397], [-4192, 8384], [281, 281], [0, -12139], [-461, 461], [-12156, 0], [342, -171], [353, 353], [15158, -7579], [482, -964], [-14496, 0], [5137, 5137], [5490, -2745], [-1635, 1635], [8334, -16668], [0, 12687], [3319, 3319], [-6054, 0], [14803, -14803], [12396, -24792], [-13516, 0], [-1669, -1669], [0, 11583], [12879, -12879], [-11636, 0], [-7108, 14216], [9033, 9033], [14908, -29816], [-1668, 0], [-13699, 13699], [0, 23173], [9593, 9593], [-32786, 16393], [3606, -7212], [25842, -12921], [0, 19827], [-9758, 4879], [2949, -2949], [12039, -12039], [0, 3363], [5865, 5865], [18169, -18169], [-9042, 0], [14390, -7195], [-2493, -2493], [18270, 0], [1882, -941], [0, 9831], [-15464, 0], [7284, -14568], [-9401, -9401], [-6239, -6239], [13615, -13615], [-5551, 5551], [-11642, 5821], [17181, 17181], [-32026, 16013], [-36430, 18215], [0, 1455], [1659, -1659], [-2542, 5084], [16571, 16571], [0, -15779], [-4246, 8492], [-17371, 17371], [-34034, 17017], [-1680, 3360], [5346, 0], [24244, 0], [18926, -37852], [-5246, 2623], [-20517, 20517], [-23852, 0], [38757, -38757], [39071, -39071], [53040, 0], [0, -367], [34566, -17283], [42166, 0], [-20220, 40440], [1501, 1501], [0, 10623], [-11662, 23324], [0, -2325], [-19017, -19017], [24654, -49308], [4843, -4843], [0, -10943]]
